%** update climate-related boundary conditions for current time step & ice surface elevation
%	Tsurface:	decreases from defined Tsealevel (may change in clim. chng. scenarios) using
%				defined lapse rate or can vary locally if initially assign
%				spatial variation in mean annual surface temperature, Ta_o
%	Adot:		this can be either a simple exponential lapse rate sort of
%	            thing or variation from modern observed accumulation rate. 
%				This is important because of situations like the low Adot
%               zone over Siple Coast... by using today's obs, includes
%               distance from moisture source


%Initial Temperature conditions

%past climate [e.g. GRIP, Byrd, Vostok, etc.]
%past sea level change [Imbrie et al., 1984]
%future climate warming [Manabe and Stouffer, 1994; IPCC, 2001; etc.]

year_old=year;

if ( (year<0) & (year>=(-250000)) )     
 del18oxyicecore=interp1(gripdatabar(:,1), gripdatabar(:,2), abs(year));
 del18oxydeepsea=interp1(specmap(:,1), specmap(:,2), abs(year));
elseif greenhouse   
% if (year>=0) & (year<=80)
%  CO2warming=warming_rate1*year;  % oC
%  del18oxyicecore=-35.27;
%  del18oxydeepsea=-1.93;
% elseif (year>80) & (year<=500)
%  CO2warming=(warming_rate1*80)+(warming_rate2*(year-80)); % oC
%  del18oxyicecore=-35.27;
%  del18oxydeepsea=-1.93;
 if (year>=0) & (year<=140)
  CO2warming=warming_rate1*year;  % oC
  del18oxyicecore=-35.27;
  del18oxydeepsea=-1.93;
 %elseif (year>140) & (year<=510)
 elseif (year>140) & (year<=1010)
  CO2warming=(warming_rate1*140)+(warming_rate2*(year-140)); % oC
  del18oxyicecore=-35.27;
  del18oxydeepsea=-1.93;
 else
  CO2warming=0;
  del18oxyicecore=-35.27;
  del18oxydeepsea=-1.93;
 end
else
 del18oxyicecore=-35.27;
 del18oxydeepsea=-1.93;
end



%Following Cuffey et. al., 1994 and Cuffey and Clow, 1997
if ( (year<=0) & (year>=(-500)) )     
cuffey_factor=2.15;
elseif ( (year<-500) & (year>=(-3000)) )
cuffey_factor=3.99;
elseif ( (year<-3000) & (year>=(-8000)) )
cuffey_factor=3.98;
elseif ( (year<-8000) & (year>=(-98000)) )
cuffey_factor=3.05;
elseif ( (year<-98000) & (year>=(-134000)) )
cuffey_factor=1.5;
elseif ( (year<-134000) )
cuffey_factor=3.05;
end



sealev=(-34.83*(del18oxydeepsea+1.93));
%%rockbelow=( (b(1:2:nods*2)<sealev) & (thick(1:2:nods*2)<=0) );

% Note the new variable Zs, which truncates the s used to compute Ts at sea level.
% Trying to use effect of bouyancy driven Ts forcing as well now, if no ice where sea, then
% set Ts at sea level Ts there... 

si=(rho_seawater-rho)/rho_seawater * thick;
mfloatclimate=((si(1:2:nods*2-1)>(s(1:2:nods*2-1)-(-34.83*(del18oxydeepsea+1.93)))) & (~barren(1:2:nods*2)));

if surfmasbal
Z=max(s(1:2:nods*2-1)-(-34.83*(del18oxydeepsea+1.93)),20*(latitude-65));  %This gives the same result as below
Zs=~mfloatclimate.*max(s(1:2:nods*2-1)-(-34.83*(del18oxydeepsea+1.93)),zeros(nods,1))+mfloatclimate.*si(1:2:nods*2-1);
Ta=Tsealevel-LapseRatemean*Z-LatTempDep*latitude + cuffey_factor*(del18oxyicecore+35.27) + CO2warming; %mean annual Temperature, Ta
Ts=Tsealevelsummer-LapseRatesummer*Zs-LatTempDepsummer*latitude + cuffey_factor*(del18oxyicecore+35.27) + CO2warming;  %summer Temperature, Ts
%%Ta=(Tnot-1.9)*(rockbelow) + Ta.*(~rockbelow);
%%Ta=(-1.9)*( (b(1:2:nods*2)<sealev) & (thick(1:2:nods*2)<=0) ) + Ta.*( (b(1:2:nods*2)>=sealev) & (thick(1:2:nods*2)>0) );
end


%%for warming or cooling entire flowline climate
%Ta=Ta-Tcool;
%Ts=Ts-Tcool;


Tsurface=Ta;

if surfmasbal
%%Positive degree day calculations...
PDD=zeros(nods,1);
timeofyear=0;
for n=1:12
timeofyear=timeofyear+30.4375;
PDD=PDD+pdd(timeofyear,Tsurface-Tnot,Ts-Tnot,sigma,nods)*30.4375;
end
PDD=PDD/sigma/sqrt(2*pi).*(PDD>0);
%When using PDD for ablation we only care about positive degree days, not
%the number of days that are below freezing.
end

%%*******For non-steady state precip over Greenland... Huybrechts et al. [1991] present precip profile
%% adjusted for changes in T associated with elevation and climate changes
Precip=Precip_o.*1.0533.^(Ta-Ta_o);
%%********

%%*******For non-steady state precip over WAIS... Huybrechts [book] present precip profile
%% adjusted for changes in T associated with elevation and climate changes
%Precip=Precip_o.*1.0616.^(Ta-Ta_o);
%Adot=FEMify*(Precip);
%%********

if surfmasbal
Sdot_nodes=Precip-0.003*PDD*1000/rho/seconds;
Sdot=FEMify*(Sdot_nodes);

soupdot_nodes=(0.003*PDD*1000/rho/seconds).*( (snow(1:2:2*nods)>0) & (0.003*PDD*1000/rho/seconds<=pmax*Precip) ) ...
+ (pmax*Precip).*( (snow(1:2:2*nods)>0) & (0.003*PDD*1000/rho/seconds>pmax*Precip) ) ...
 - (0.008*PDD*1000/rho/seconds).*( (snow(1:2:2*nods)==0) & (soup(1:2:2*nods)>=0) );
soupdot=FEMify*( soupdot_nodes );

%The below follows the logic that we are looking for... Sdot and soupdot, although they are
%in ice equivalent accum rates, are not ice-sheet ice accumulating, and are therefore
%not considered a part of Adot until the proper conditions are met (full firnlayer thickness of snow
%exists)

Adot=FEMify*( Sdot_nodes.*(Sdot_nodes>=0 & snow(1:2:2*nods)>=firnlayer) ... 
    -(ablationfactor*0.008*PDD*1000/rho/seconds).*( (snow(1:2:2*nods)==0) & (soup(1:2:2*nods)==0) ) );
end