%The PSU/UofC flowline model (B.R. Parizek, D.R. MacAyeal, and R.B. Alley
%with contributions from P.M. Cutler and C.L. Hulbe)...
%[Parizek, 2000; Cutler et al., 2000; Cutler et al., 2001; Parizek et al., 2002;
%Parizek et al., 2003; Parizek, 2003; Parizek and Alley, 2004;
%Parizek and Alley, 2004; Parizek et al., 2005]

clear all
nsteps=30;  
stepsremaining=nsteps;
prelimpsu
dt_years=10;  
dt=dt_years*seconds;
asynchstep=10;
dtmass=dt/asynchstep;
year=-590000;  % want at least 590ka because it takes ~170ka to reach thermal equilibrium and want climate from 420ka to present to influence the ice sheet

snapshottimes=[-80 -65 -60 -55 -50 -45 -42 -30 -21 -9]*1e3; 
snaptime=snapshottimes(snapcount+1);

matrixpsu	
initializepsu

movie_frame=10; %make a frame every movie_frame # of steps
frames_2_make=fix(nsteps/movie_frame);
movie_count=movie_frame-1;
frame_count=1;


% Time-stepping portion of code:

massstep=1;  % 0 supresses time stepping of ice thickness, zero for ice free testing
thermstep=1;  % 0 supresses time stepping of thermodynamics
advection=1;   % 0 supresses inclusion of horizontal advection in the thermodynamics, zero for ice free testing
surfmasbal=1;  % 0 supresses inclusion of firn and superimposed ice dynamics and their inclusion in Adot calcs
greenhouse=0;  % 0 supresses inclusion of greenhouse warming in year>0 simulations

%comment out the below two lines when restarting runs from a saved file
flowlawpsu    
compvelspsu  

	tic
for timestep=1:nsteps

if  year<-420000
	dt_years=10;
	dt=dt_years*seconds;
	asynchstep=10;
	dtmass=dt/asynchstep;
elseif year>=-420000
	dt_years=5;
	dt=dt_years*seconds;
	asynchstep=1;
	dtmass=dt/asynchstep;
end


Vinitialpsu   % for steady state run a la EISMINT

% saving output at specified years... at times saving all variables
if year==-590000
save climaterun_590000.mat
end
if year==-575000
save climaterun_575000.mat year Peclet_num w timestep s b Sdot soupdot Adot Bdot sealev udefnode uslidenode Vertvelnode T thick snow soup barren nosnow nosoup melted polythermal del18oxydeepsea dT_Vostok
end
%etc...

 if year==snaptime	
    snapfile = 'snap' ;	
    save([snapfile int2str(snapcount+1)], 'year', 'Anode', 'Peclet_num', 'w', 'timestep', 's', 'b', 'Sdot', 'soupdot', 'Adot', 'Bdot', 'sealev', 'udefnode', 'uslidenode', 'Vertvelnode', 'T', 'thick', 'snow', 'soup', 'barren', 'nosnow', 'nosoup', 'melted', 'polythermal', 'dT_Vostok', 'del18oxydeepsea');
	

    snapcount=snapcount+1;
    if snapcount<=9
       snaptime=snapshottimes(snapcount+1);
    else 
       snaptime=999999;
    end
 end


 if massstep     % Mass balance step
  for astep=1:asynchstep
climatepsu
basalmeltingpsu
masbalpsu
flowlawpsu
compvelspsu
if surfmasbal
snowbalpsu
soupbalpsu
end
end
end

 if thermstep % Thermodynamics
climatepsu
basalmeltingpsu
verticalpsu
end
flowlawpsu
compvelspsu   %need this here to update u_total b/c Anode, Inode, etc change with Temp
isostasypsu	%using elastic lithosphere, relaxation asthenosphere

year=year+dt_years;
stepsremaining=stepsremaining-1;
maxs=max(s(1:2:nods*2-1));
maxthick=max(thick(1:2:nods*2-1));
time_year=[timestep stepsremaining year maxs maxthick]
save runstatus time_year -ascii;


%Not for running in background
movie_count=movie_count+1;
if (movie_count==movie_frame)
  movieline
  moviecolor
  movievelocity
  movie_count=0;
  frame_count=frame_count+1;
end


deltaVpsu   %for steadystate a la EISMINT
tvpsu     %generating matrix of important temporal and spatial output variables
sitepsu     %generating matrix of important temporal output variables at a specified location

if firstsstime
  if sstime==10000     %Steady State Criterium
   simulatedtime=year
   save Volume_steadystate.mat
   firstsstime=0;
  end
end


 
end % end time step

save filename.mat   %specify a filename to save all variables after the final timestep

time_run=toc/60/60

figure(20),movie(Mline,1,1)
figure(121),movie(Mcolor,1,1)
figure(34),movie(Mvelocity,1,1)


