% This code does the isostatic diffusion step for the full region (inland ice, ice shelf and ice stream)
%   sealev:     allowing SL to affect b wrt load of sea water when appropriate in isostasy... 


%added to follow SL-fluctuation reference frame...
biso=b(1:2:nods*2)-sealev;


% Is a particular node floating?  Used in isostasy calculations...
si=(rho_seawater-rho)/rho_seawater * thick;
mfloat=((si(1:2:nods*2)>s(1:2:nods*2)) & (~barren(1:2:nods*2)));

grav_load= (rho*thick(1:2:nods*2)).*(~mfloat) ...
  - (rho_seawater*biso).*(mfloat) ...  %b-->biso added to follow SL-fluctuation reference frame...
  - (rho_seawater*biso.*(biso<=0 & barren(1:2:nods*2))) ;  %b-->biso added to follow SL-fluctuation reference frame... 
grav_load=FEMify*grav_load;

%********  USE THE FOLLOWING CODE FOR ELASTIC LITHOSPHERE, RELAXATION ASTHENOSPHERE *****
A=sparse(rowperm,colperm,Dr*Ggauss,nods*2,nods*2);
%R=rho*g*Aperm*thick;  %used for "air-filled" ocean
R=g*Aperm*grav_load;   %used to take into acount "ocean-water-filled" ocean
Aprime=PLI*(rhoasth*g*Aperm+A)*PLI';

%%***The below is to assign non-zero deflection values at endpts of boundary domain
deflect_spec=zeros(nods*2,1);
deflect_spec(1)=grav_load(1)/rhoasth;   % If we wanted to specify deflection as nonzero at west end
deflect_spec(nods*2-1)=grav_load(nods*2-1)/rhoasth;  % If we wanted to specify deflection as nonzero at east end
deflect_spec=deflect_spec.*(~mls);

Rprime=PLI*(R-(rhoasth*g*Aperm+A)*deflect_spec); %For divides (fixed zero slope) and non-zero thickness boundary conditions (works for zero-valued cases also)

lith_prime=Aprime\Rprime;

lith_deflection=PLI'*lith_prime + deflect_spec;  % Note: look out for divides.  Putting known BCs back into soln.
%****

%% perform relaxation: 
Rprime=Aperm*(b/dt-((lith_deflection-bnot+b)/tau_asthenosphere));
Aprime=(1/dt)*Aperm;
b=Aprime\Rprime;
%********  END ELASTIC LITHOSPHERE, RELAXATION ASTHENOSPHERE ****************************

% update s:

s=b+thick;


% Is a particular node floating?  Only purpose here is for
% graphics... would be updated in masbalpsu01 before computing
% isostatic rebound next time... so not needed for isos calcs.
si=(rho_seawater-rho)/rho_seawater * thick;
mfloat=((si(1:2:nods*2)>s(1:2:nods*2)) & (~barren(1:2:nods*2)));

