% Computes integrals of basis functions that result from
% weighted residual method and FE representation of the
% variable and test function.
% Sets up the Agauss, Bgauss, Rgauss, and PDI matrices.

gamma=zeros(nel,4);
alpha=zeros(nel,4);
beta=zeros(nel,4);
delta=zeros(nel,4);

%Solving the matrix equation XU=I (where U=invX), in 
%order to obtain the coeffs to the higher order 
%Hermite Basis Functions... 


%The below should produce s2 and s4 that are d/dx

for n=1:nel
 X=[1 0 0 0
 0 1/Le(n) 0 0 
 1 1 1 1
 0 1/Le(n) 2/Le(n) 3/Le(n)];

 U=inv(X);
 gamma(n,:)=U(1,:);
 alpha(n,:)=U(2,:);
 beta(n,:)=U(3,:);
 delta(n,:)=U(4,:);
end


% Gaussian quadrature stuff:
%Adjust for the fact that we are integrating over a
%domain that is half as large.  To do this, we need 
%to divide the GaussWeights by 2 because they are 
%defined for a domain size from -1 to 1.  
%For snowbal, soupbal, and tillbal we need 5 pt quadrature for 8th
%degree polynomials (k <= 2(n_g)-1):

GaussWeights=.5*[.3478548451 .6521451548 .6521451548 .3478548451]';
D_GaussWeights=.5*[.2369268851 .4786286705 .5688888888 .4786286705 .2369268851]';


%By definition of the Gaussian Quadrature Formula, xi
%goes from -1 to 1.  Therefore, because my domain is
%from 0 to 1, I will have different gauss abscissae 
%and weight coefficients.  Below we convert to
%xi from 0 to 1.
tablexi=[-.8611363115 -.3399810435 .3399810435 .8611363115]';
D_tablexi=[-.9061798459 -.5384693101 0.0 .5384693101 .9061798459]';
xig=zeros(4,1); %Included so that matrix dimensions agree below
D_xig=zeros(4,1); 

for gauss=1:4
 xig(gauss)=(tablexi(gauss)+1)/2;
end
for gauss=1:5
 D_xig(gauss)=(D_tablexi(gauss)+1)/2;
end

Agauss=zeros(nel*4,4);
Bgauss=Agauss;
Cgauss=Agauss;
D1gauss=Agauss;
D2gauss=Agauss;
D3gauss=Agauss;
D4gauss=Agauss;
F1gauss=Agauss;
F2gauss=Agauss;
F3gauss=Agauss;
F4gauss=Agauss;
E1gauss=Agauss;  %for ELRA isostacy
E2gauss=Agauss;  %for ELRA isostacy
E3gauss=Agauss;  %for ELRA isostacy
E4gauss=Agauss;  %for ELRA isostacy
Ggauss=Agauss;   %for ELRA isostacy


rowperm=zeros(nel*4,4);
colperm=zeros(nel*4,4);

row=zeros(4,4);
col=zeros(4,4);

A=zeros(4,4);
B=A;
C=A;
D1=A;
D2=A;
D3=A;
D4=A;
F1=A;
F2=A;
F3=A;
F4=A;
E1=A;
E2=A;
E3=A;
E4=A;
G=A;


for n=1:nel
 for i=1:4
  for j=1:4
	  
   A(i,j)=sum(GaussWeights(:).*...
   (gamma(n,i)+alpha(n,i)*xig+...
   beta(n,i)*xig.^2+delta(n,i)*xig.^3).*...
   (gamma(n,j)+alpha(n,j)*xig+...
   beta(n,j)*xig.^2+delta(n,j)*xig.^3));

   B(i,j)=sum(GaussWeights(:).*...
   (alpha(n,i)+2*beta(n,i)*xig+...
   3*delta(n,i)*xig.^2).*...
   (alpha(n,j)+2*beta(n,j)*xig+...
   3*delta(n,j)*xig.^2));

   C(i,j)=sum(GaussWeights(:).*...
   (gamma(n,i)+alpha(n,i)*xig+...
   beta(n,i)*xig.^2+delta(n,i)*xig.^3).*...
   (alpha(n,j)+2*beta(n,j)*xig+...
   3*delta(n,j)*xig.^2));

   D1(i,j)=sum(D_GaussWeights(:).*...
   (gamma(n,1)+alpha(n,1)*D_xig+...
   beta(n,1)*D_xig.^2+delta(n,1)*D_xig.^3).*...
   (alpha(n,i)+2*beta(n,i)*D_xig+...
   3*delta(n,i)*D_xig.^2).*...
   (gamma(n,j)+alpha(n,j)*D_xig+...
   beta(n,j)*D_xig.^2+delta(n,j)*D_xig.^3));

   D2(i,j)=sum(D_GaussWeights(:).*...
   (gamma(n,2)+alpha(n,2)*D_xig+...
   beta(n,2)*D_xig.^2+delta(n,2)*D_xig.^3).*...
   (alpha(n,i)+2*beta(n,i)*D_xig+...
   3*delta(n,i)*D_xig.^2).*...
   (gamma(n,j)+alpha(n,j)*D_xig+...
   beta(n,j)*D_xig.^2+delta(n,j)*D_xig.^3));
   
   D3(i,j)=sum(D_GaussWeights(:).*...
   (gamma(n,3)+alpha(n,3)*D_xig+...
   beta(n,3)*D_xig.^2+delta(n,3)*D_xig.^3).*...
   (alpha(n,i)+2*beta(n,i)*D_xig+...
   3*delta(n,i)*D_xig.^2).*...
   (gamma(n,j)+alpha(n,j)*D_xig+...
   beta(n,j)*D_xig.^2+delta(n,j)*D_xig.^3));

   D4(i,j)=sum(D_GaussWeights(:).*...
   (gamma(n,4)+alpha(n,4)*D_xig+...
   beta(n,4)*D_xig.^2+delta(n,4)*D_xig.^3).*...
   (alpha(n,i)+2*beta(n,i)*D_xig+...
   3*delta(n,i)*D_xig.^2).*...
   (gamma(n,j)+alpha(n,j)*D_xig+...
   beta(n,j)*D_xig.^2+delta(n,j)*D_xig.^3));
   
   

   
   F1(i,j)=sum(GaussWeights(:).*...
   (gamma(n,1)+alpha(n,1)*xig+...
   beta(n,1)*xig.^2+delta(n,1)*xig.^3).*...
   (alpha(n,i)+2*beta(n,i)*xig+...
   3*delta(n,i)*xig.^2).*...
   (alpha(n,j)+2*beta(n,j)*xig+...
   3*delta(n,j)*xig.^2));

   F2(i,j)=sum(GaussWeights(:).*...
   (gamma(n,2)+alpha(n,2)*xig+...
   beta(n,2)*xig.^2+delta(n,2)*xig.^3).*...
   (alpha(n,i)+2*beta(n,i)*xig+...
   3*delta(n,i)*xig.^2).*...
   (alpha(n,j)+2*beta(n,j)*xig+...
   3*delta(n,j)*xig.^2));
   
   F3(i,j)=sum(GaussWeights(:).*...
   (gamma(n,3)+alpha(n,3)*xig+...
   beta(n,3)*xig.^2+delta(n,3)*xig.^3).*...
   (alpha(n,i)+2*beta(n,i)*xig+...
   3*delta(n,i)*xig.^2).*...
   (alpha(n,j)+2*beta(n,j)*xig+...
   3*delta(n,j)*xig.^2));
   
   F4(i,j)=sum(GaussWeights(:).*...
   (gamma(n,4)+alpha(n,4)*xig+...
   beta(n,4)*xig.^2+delta(n,4)*xig.^3).*...
   (alpha(n,i)+2*beta(n,i)*xig+...
   3*delta(n,i)*xig.^2).*...
   (alpha(n,j)+2*beta(n,j)*xig+...
   3*delta(n,j)*xig.^2));
	  
   G(i,j)=sum(GaussWeights(:).*...
   (2*beta(n,i)+6*delta(n,i)*xig).*...
   (2*beta(n,j)+6*delta(n,j)*xig));

   
  end
 end

E1=D1';
E2=D2';
E3=D3';
E4=D4';

 
 A=Le(n).*A;
 Agauss( ((n-1)*4+1):(n*4),1:4) =A;

 row=[index2(n,1:4)]'*ones(1,4);
 col=ones(4,1)*[index2(n,1:4)];


 rowperm( ((n-1)*4+1):(n*4),1:4) = row;
 colperm( ((n-1)*4+1):(n*4),1:4) = col;

 B=1/Le(n).*B;
 Bgauss( ((n-1)*4+1):(n*4),1:4) =B;

 Cgauss( ((n-1)*4+1):(n*4),1:4) =C;

 D1gauss( ((n-1)*4+1):(n*4),1:4) =D1;
 D2gauss( ((n-1)*4+1):(n*4),1:4) =D2;
 D3gauss( ((n-1)*4+1):(n*4),1:4) =D3;
 D4gauss( ((n-1)*4+1):(n*4),1:4) =D4; 
 
 E1gauss( ((n-1)*4+1):(n*4),1:4) =E1;
 E2gauss( ((n-1)*4+1):(n*4),1:4) =E2;
 E3gauss( ((n-1)*4+1):(n*4),1:4) =E3;
 E4gauss( ((n-1)*4+1):(n*4),1:4) =E4; 


 F1=1/Le(n).*F1;
 F2=1/Le(n).*F2;
 F3=1/Le(n).*F3;
 F4=1/Le(n).*F4;
 F1gauss( ((n-1)*4+1):(n*4),1:4) =F1;
 F2gauss( ((n-1)*4+1):(n*4),1:4) =F2;
 F3gauss( ((n-1)*4+1):(n*4),1:4) =F3;
 F4gauss( ((n-1)*4+1):(n*4),1:4) =F4; 

 G=1/(Le(n)^3).*G;
 Ggauss( ((n-1)*4+1):(n*4),1:4) =G;


end


%Set up parsing matrix...

row=zeros(nel*4*4,1);
col=zeros(nel*4*4,1);
value=ones(nel*4*4,1);

count=1-nel;
for i=1:4
 for j=1:4
  count=count+nel;
  row(count:count+nel-1)=index2(:,i);
  col(count:count+nel-1)=index2(:,j);
 end
end
Connection=sparse(row,col,value,nods*2,nods*2);
numnz=nnz(Connection)
clear row col value Connection

%Create a mask: 1=active vars, 0=specified

masknodalinfopsu
PDI=sparse([1:nactive]',find(ms==1)',ones(nactive,1),nactive,nods*2);	
%PII=PDI;
% % For LLDA isostacy
% %nactive=sum(mlls);
% %PII=sparse([1:nactive]',find(mlls==1)',ones(nactive,1),nactive,nods*2);
%For ELRA isostacy
nactive=sum(mls);	
PLI=sparse([1:nactive]',find(mls==1)',ones(nactive,1),nactive,nods*2);


% Compute total area, etc.

totlength=zeros(nods,1);
for n=1:nel
 totlength(n)=totlength(n)+Le(n);
 totlength(n+1)=totlength(n+1)+Le(n);
end

row=zeros(4,1);
col=zeros(4,1);

%The below six lines produce the same result as: Aperm=sparse(rowperm,colperm,Agauss,nods*2,nods*2);
Aperm=spalloc(nods*2,nods*2,numnz);
for n=1:nel
 row=[index2(n,1:4)]';
 col=row;
 Aperm(row,col)=Aperm(row,col)+Agauss(((n*4)-3):(n*4),:);
end

expand_by_4=spalloc(nel*4,nel,nel*4);
for n=1:nel
 expand_by_4((n-1)*4+1:n*4,n)=ones(4,1);
end

% The following matrix will create a field of nodal variables from information
% about the field value at each node. In other words, 
%
%                    FEMify*field=nodalvariables
%
% where field is, say, initial data on each node, and nodalvariables are 
% the column vector of length 2*nods which, when multiplied appropriately
% by interpolation functions, yields the FEM representation of the field.
% Note this doesn't give an accurate value at ice sheet margins that fall
% within the limits of the grid.



FEMify=spalloc(2*nods,nods,3*nods);

for n=1:nods
 FEMify(2*n-1,n)=1;
end
FEMify(2,1)=-1/alpha(1,2);
FEMify(2,2)=1/alpha(1,2);
for n=2:nods-1
 FEMify(n*2,n-1)=-1/(Le(n-1)+Le(n))*Le(n)/alpha(n,2);
 FEMify(n*2,n+1)=1/(Le(n-1)+Le(n))*Le(n)/alpha(n,2);
end
FEMify(nods*2,nods-1)=-1/(alpha(nel,4)+2*beta(nel,4)+3*delta(nel,4));
FEMify(nods*2,nods)=1/(alpha(nel,4)+2*beta(nel,4)+3*delta(nel,4));


%note changes****
UPWINDddx_p=spalloc(nods,nods,2*nods); % Note this doesn't give an accurate value at ice sheet margins that fall within the limits of the grid
for n=2:nods-1
 UPWINDddx_p(n,n)=1/Le(n-1);
 UPWINDddx_p(n,n-1)=-1/Le(n-1);
end
UPWINDddx_p(nods,nods)=1/Le(nel);
UPWINDddx_p(nods,nods-1)=-1/Le(nel);
UPWINDddx_m=spalloc(nods,nods,2*nods); % Note this doesn't give an accurate value at ice sheet margins that fall within the limits of the grid
for n=2:nods-1
 UPWINDddx_m(n,n)=-1/Le(n);
 UPWINDddx_m(n,n+1)=1/Le(n);
end
UPWINDddx_m(1,1)=-1/Le(1);
UPWINDddx_m(1,2)=1/Le(1);
%****

clear A B C D1 D2 D3 D4 E1 E2 E3 E4 F1 F2 F3 F4 G X
AVGx=spalloc(nel,nods,2*nel); 
for n=1:nel
AVGx(n,n)=1/2;
AVGx(n,n+1)=1/2;
end
AVGz=spalloc(nvel,nlev,2*nvel); 
for n=1:nvel
AVGz(n,n)=1/2;
AVGz(n,n+1)=1/2;
end


Alocation=[1:nlev:(nlev*nods)]';

Blocation=[rocklev+1:nlev:(nlev*nods+rocklev)]';


T_tentative=zeros(nlev,nods);

zeta=[vnodal_depths zetaice(2:icelev)']';
al=zeros(nvel,2);
dl=zeros(nvel,1);
for n=1:nvel
X=[1 zeta(n) 
1 zeta(n+1)];
phi=inv(X);
al(n,:)=phi(2,:);
dl(n)=zeta(n+1)-zeta(n);
end

%note changes****
checkrock=(rocklev>0);  %added for no rock runs.
if checkrock
for n=1:rocklev
depth(n) = -(vnodal_depths(n) + vnodal_depths(n+1))/2;				%z-coord (from bed surface), for PMP
end
else
depth=0;
end
%*****

thm_diff=krock/rhorock/caprock*ones(nvel,1);
for n=rocklev+1:nvel
thm_diff(n)= kiceconstant/rho/ciceconstant;
end
thm_cond=krock*ones(nvel,1);
for n=rocklev+1:nvel
thm_cond(n)= kiceconstant;
end
thm_cap=rhorock*caprock*ones(nvel,1);
for n=rocklev+1:nvel
thm_cap(n)= rho*ciceconstant;
end


rowvertperm=zeros(nvel,4*nods);
colvertperm=zeros(nvel,4*nods);
valuevertperm1=zeros(nvel,4);
valuevertperm21=zeros(nvel,4);
valuevertperm22=zeros(nvel,4);
valuevertperm3=zeros(nvel,4);


for n=1:nvel


rowvertperm(n,1:nods)=lindex(n,1)+[0:nlev:(nlev*nods)-1];
colvertperm(n,1:nods)=lindex(n,1)+[0:nlev:(nlev*nods)-1];

rowvertperm(n,nods+1:2*nods)=lindex(n,1)+[0:nlev:(nlev*nods)-1];
colvertperm(n,nods+1:2*nods)=lindex(n,2)+[0:nlev:(nlev*nods)-1];

rowvertperm(n,2*nods+1:3*nods)=lindex(n,2)+[0:nlev:(nlev*nods)-1];
colvertperm(n,2*nods+1:3*nods)=lindex(n,1)+[0:nlev:(nlev*nods)-1];

rowvertperm(n,3*nods+1:4*nods)=lindex(n,2)+[0:nlev:(nlev*nods)-1];
colvertperm(n,3*nods+1:4*nods)=lindex(n,2)+[0:nlev:(nlev*nods)-1];

valuevertperm1(n,1)=dl(n)/3/dt*thm_cap(n);
valuevertperm21(n,1)=dl(n)*al(n,1)/3*thm_cap(n);
valuevertperm22(n,1)=dl(n)*al(n,1)/6*thm_cap(n);
valuevertperm3(n,1)=dl(n)*thm_cond(n)*al(n,1)*al(n,1);

valuevertperm1(n,2)=dl(n)/6/dt*thm_cap(n);
valuevertperm21(n,2)=dl(n)*al(n,2)/3*thm_cap(n);
valuevertperm22(n,2)=dl(n)*al(n,2)/6*thm_cap(n);
valuevertperm3(n,2)=dl(n)*thm_cond(n)*al(n,2)*al(n,1);

valuevertperm1(n,3)=dl(n)/6/dt*thm_cap(n);
valuevertperm21(n,3)=dl(n)*al(n,1)/6*thm_cap(n);
valuevertperm22(n,3)=dl(n)*al(n,1)/3*thm_cap(n);
valuevertperm3(n,3)=dl(n)*thm_cond(n)*al(n,1)*al(n,2);

valuevertperm1(n,4)=dl(n)/3/dt*thm_cap(n);
valuevertperm21(n,4)=dl(n)*al(n,2)/6*thm_cap(n);
valuevertperm22(n,4)=dl(n)*al(n,2)/3*thm_cap(n);
valuevertperm3(n,4)=dl(n)*thm_cond(n)*al(n,2)*al(n,2);


end



wbar=zeros(nvel,nods);
qbar=zeros(nvel,nods);

Peclet_num=zeros(1,nods);

u_total=zeros(nods,icelev);
u_bar=zeros(nods,1);
meltedbar=zeros(nel,1);


%note changes
dzeta=zeros(icelev-1,1);
for lev=1:icelev-1
dzeta(lev)=(zetaice(lev+1)-zetaice(lev)); % note that this is an average over the element
end

Geoflux_vector=zeros(nods*nlev,1);
