% Preliminary code set-up 
%***** Choose a flowline for initialization

load griptransect.mat  %(X,Y,accum,b,h,lat,lon,s)
nods=length(s);
nel=nods-1;
Xposition=X;
Yposition=Y;
x=zeros(nods,1);
x=linspace(0,nel*20e3,nods)';
bedrockelev=b;
surfaceelev=s;
icethickness=surfaceelev-bedrockelev;
%icethickness=h;
latitude=lat;
longitude=lon;

gripnode=find(x==920000);
westsea=find(x==400000);
eastsea=find(x==1240000);
n=find(bedrockelev<=-395);
icedivide_o=find(surfaceelev==max(surfaceelev));
activewest=n(find(icedivide_o-n.*(n<=icedivide_o)==min(icedivide_o-n.*(n<=icedivide_o))));
activeeast=n(find(n==activewest)+1);



Xcontr = [Xposition(30) Xposition(60)];	%controls position of bedrock transition for sliding
seconds=31556926;  %seconds in a year

%%*****
%%***** For use with Huybrechts et al [1991] Greenland precip calcs
Precip_o=accum/seconds;    
del18oxyicecore=-35.27;
del18oxydeepsea=-1.93;
sealev=(-34.83*(del18oxydeepsea+1.93));
%rockbelow=( (bedrockelev<sealev) & (icethickness<=0) );
barren=zeros(nods*2,1);
barren(1:2:nods*2-1)=(icethickness<=0);
barren(4:2:nods*2-2)=( barren(1:2:nods*2-5) & barren(5:2:nods*2-1) );
barren(2)=barren(3);
barren(nods*2)=barren(nods*2-3);
barren(2:2:nods*2)=barren(2:2:nods*2).*barren(1:2:nods*2-1); %added to take care of 3-noded island of ice... 1,0  0,1  1,0 would result when want 1,0 0,0 1,0
rho_seawater=1028;
rho=917;  
Tnot=273.15;     
Tsealevel=Tnot+49.13;  
Tsealevelsummer=Tnot+30.78; 
LapseRatemean=0.007992; 
LapseRatesummer=0.006277; 
LatTempDep=0.7576;     
LatTempDepsummer=0.3262;  
zsealevel=0;          
cuffey_factor=1.5 ; 
CO2warming=0;
si=(rho_seawater-rho)/rho_seawater * icethickness;
mfloatclimate=((si>(surfaceelev-(-34.83*(del18oxydeepsea+1.93)))) & (~barren(1:2:nods*2)));
Z=max(surfaceelev-(-34.83*(del18oxydeepsea+1.93)),20*(latitude-65));  %This gives the same result as below
Zs=~mfloatclimate.*max(surfaceelev-(-34.83*(del18oxydeepsea+1.93)),zeros(nods,1))+mfloatclimate.*si;
Ta=Tsealevel-LapseRatemean*Z-LatTempDep*latitude + cuffey_factor*(del18oxyicecore+35.27) + CO2warming; %mean annual Temperature, Ta
Ts=Tsealevelsummer-LapseRatesummer*Zs-LatTempDepsummer*latitude + cuffey_factor*(del18oxyicecore+35.27) + CO2warming;  %summer Temperature, Ts
Ta_o=Ta;
s=zeros(nods*2,1);
s(1:2:nods*2)=surfaceelev;



diffbar=zeros(nel,1);
Le=zeros(nel,1);
index2=zeros(nel,4);
index=zeros(nel,2);
for n=1:nel
 index2(n,:)=[n*2-1 n*2 (n+1)*2-1 (n+1)*2];
 index(n,:)=[n n+1];
end
Le=[x(index(:,2))-x(index(:,1))];

%................ vertical resolution...............%

%for (sliding) runs through glacial-interglacial climate

icelev=70;   
rockscale=800;    %this is the scale chosen for "deep rock" (100 m spacing)
sedrock_scale=300;  %this is the scale chosen for "sedimentary rock" thickness (10 m spacing)
quaternary_scale=20; % surficial unlithified sediment  (2 m spacing)
rocklev=fix(quaternary_scale/2)+fix(sedrock_scale/10) + fix(rockscale/100);  %want total depth between ~500-1000m for sliding =1120m here ~40ka.
vnodal_depths=[linspace(-(rockscale+sedrock_scale+quaternary_scale),-(sedrock_scale+quaternary_scale),fix(rockscale/100)+1) ...
linspace(-(sedrock_scale+quaternary_scale)+10,-(quaternary_scale),fix(sedrock_scale/10)) ...
linspace(-(quaternary_scale)+2,0,fix(quaternary_scale/2)) ]; 
nlev=icelev+rocklev;
lowlayers=20;
lindex=[linspace(1,nlev-1,nlev-1) 
linspace(2,nlev,nlev-1)]';
nvel=nlev-1;
dzrock=[vnodal_depths(lindex(1:rocklev,2))-vnodal_depths(lindex(1:rocklev,1))];

%Note: the below zetaice has been corrected to get rid of the uncosmetic gap.
dummylow=logspace(-2,-1,lowlayers);
zetaice=[dummylow-dummylow(1)...
linspace(dummylow(lowlayers)-dummylow(lowlayers-1)...
+dummylow(lowlayers)-dummylow(1),1,icelev-lowlayers) ]';



%for no rock runs
%
%icelev=100;
%rockscale=0;    %this scale chosen for "deep rock"
%sedrock_scale=0;  %this is the scale chosen for "sedimentary rock" thickness
%quaternary_scale=0; % surficial unlithified sediment
%rocklev=0; 
%vnodal_depths=0;   
%nlev=icelev+rocklev;
%lowlayers=20;
%lindex=[linspace(1,nlev-1,nlev-1) 
%linspace(2,nlev,nlev-1)]';
%nvel=nlev-1;
%dzrock=1; %will be dividing by 1... 
%
%%zetaice=linspace(0,1,icelev)';  %linear spacing in ice
%
%%Note: the below zetaice has been corrected to get rid of the uncosmetic gap.
%dummylow=logspace(-2,-1,lowlayers);
%zetaice=[dummylow-dummylow(1)...
%linspace(dummylow(lowlayers)-dummylow(lowlayers-1)...
%+dummylow(lowlayers)-dummylow(1),1,icelev-lowlayers) ]';
%
%
%.......................................






%%Model constants


E=3.0;                  %"enhancement factor" added to flow law for glac/intergl ice
seconds=31557600;        %number of seconds in one year (seconds=60*60*24*365.25) 
g=9.81;                 %acceleration due to gravity, m/s^2
rho=917;                %density of ice, kg/m^3
rho_seawater=1028;       %density of sea water, kg/m^3
rhorock=2700;           %density of bedrock, kg/m^3
rhoasth=3300;           %density of asthenosphere, kg/m^3
rhowater=1000;          %density of fresh water, kg/m^3
Rgas=8.314;             %gas constant, J/(mol K)
Ao=1e-16/seconds;       %flow-law "softness" parameter, Pa^-3 s^-1
aflowcold=1.14e-5/seconds; %cold rate constant for T<263.15, Pa^-3 s^-1
aflowhot=5.47e10/seconds;  %warm rate constant for T>=263.15, Pa^-3 s^-1
Tdiscrim=263.15;        %temperature in flow law, K   (-10oC) 
Qcold=60e3;             %cold creep activation energy for T<263.15, J/mol
Qhot=139e3;             %warm creep activation energy for T>263.15, J/mol
Qvapor=51e3;			%equilibrium vapor pressure activation energy, J/mol
Phi=8.7e-4/(rho*g);     %9.74e-8 K/Pa;  pressure dep.of melt temp, K Pa^-1 

%**************
% Various Sliding parameters...
%C_sliding=5e-3/seconds;	%EISMINT Level 3
C_sliding=1.3*2.02e-4/seconds;
C_sliding1=.5e-3/seconds;	%variable sliding dependent on rock type (low value for rough inland bed)
C_sliding2=10e-3/seconds;	%variable sliding dependent on rock type (high value for lubricated till)
C_sliding3=100e-3/seconds;

kiceconstant=2.1;       %thermal conductivity of ice, J m^-1 K^-1 s^-1
ciceconstant=2009;      %specific heat capacity of ice, J kg^-1 K^-1
Geoflux=0.05;          %geothermal flux for Canadian rocks, W/m^2  (Huybrechts-- 54.6 mW/m^2)
krock=3;                %thermal conductivity of rock (from Huybrechts), J m^-1 K^-1 s^-1
caprock=1000;        %specific heat capacity of rock (from Huybrechts), J kg^-1 K^-1
Da=1e8/seconds;         %diffusivity of the asthenosphere, m^2/s
Dr=1e25;                % flexural rigidity (LeMeur and Huybrechts), Nm
tau_asthenosphere= 3000*seconds; % asthenospheric relaxation constant (LeMeur and Huybrechts), s
fusion=3.35e5;          %3.33e5 in Serway? latent heat of fusion of water, Joule/kg
Tnot=273.15;     %freezing temperature of water at standard pressure, K
Tsealevel=Tnot+49.13;  %temperature at sea level(EISMINT) [~-5oC today Greenland when accounting for LatTempDep & lat], K
Tsealevelsummer=Tnot+30.78; %summer temp at sea level (EISMINT) [~7oC today Greenland when accounting for LatTempDepsummer & lat], K NOTE:was 30.78
LapseRatemean=0.007992; %atmospheric temp lapse rate, K/m (annual ave)
LapseRatesummer=0.006277; %summer atm. temp. lapse rate, K/m
LatTempDep=0.7576;      %mean latitude/temperature dependence, K/deglat
LatTempDepsummer=0.3262;  %summer lat/temp dependence, K/deglat
zsealevel=0;            %sealevel modification from present, m
firnlayer=45;            %firn layer thickness in ice equivalent, m
souplayer=15;            %maximum layer of superimposed ice, m
%firnlayer=15;            %used for EISMINT... change? firn layer thickness in ice equivalent, m
%souplayer=5;            %used for EISMINT... change?  maximum layer of superimposed ice, m
pmax=0.6;                %runoff parameter for superimposed ice
sigma=5;        %standard deviation of daily temperature (Huybrechts, paleo^3)
K=0;           % Hydraulic Conductivity of Till/Film flow system, m/s
Ep=0.95;          % pressure head parameter 
Tcool=0;         % Fudge factor to make an ice sheet's climate cooler
melttolerance=-.01;  % tolerance for melted T at bed in verticalpsu01.m subroutine... affects melted, Bdot, etc.
SLAR=0.65/seconds;  % See Tony Payne's Paper based on Huybrecths [1986] for Antarctica, Sea Level Accumulation Rate, m/s
desert_effect=2e-4/seconds; % See Tony Payne's Paper, (Huybrechts, 1986)
cuffey_factor=2.25;   % Kurt Cuffey et al.'s oxygen isotope paleothermometer
ablationfactor=1; %1: typical ice ablation rate, 1.5: when Adot<0, ice ablates at 1.5X normal rate {climatepsu} 



kice=zeros(nods,icelev);
capice=zeros(nods,icelev);
Adot=zeros(nods*2,1);
Bdot=zeros(nods*2,1);
Tdot=zeros(nods*2,1);
melted=zeros(nods,1); % initially frozen
barren=zeros(nods*2,1);
nosnow=zeros(nods*2,1);
nosoup=zeros(nods*2,1);
notill=zeros(nods*2,1);
mfloat=zeros(nods,1);
nowater=zeros(nods*2,1);

bi=zeros(nods*2,1);
si=zeros(nods*2,1);


% Velocity initialization:

I=zeros(nel,icelev);  % becomes defm vel
Inode=zeros(nods,icelev);
Ibar=zeros(nel,icelev);
grads2=zeros(nel,1);
grads2node=zeros(nods,1);
udefnode=zeros(nods,icelev);
udefbar=zeros(nel,icelev);
uslidebar=zeros(nel,1);
uslidenode=zeros(nods,1);
u=zeros(nods,1);
div=zeros(nel,icelev); % calculated from high-order surface parameterization
divnode=zeros(nods,icelev);
Qnode=zeros(nods,icelev);    %becomes strain heating with other stuff in it
W_strainheatnode=zeros(nods,icelev); %becomes strain heating
Anode=zeros(nods,icelev);    %matrix with flow law softness parameter


% Allocate space for temperature conditions:

T=zeros(nods,nlev);

%initial temps such that bed is at Tpmp and linear gradient up to Tai...
T(:,rocklev+1)=Tnot-rho*g*icethickness*Phi;

Tsurface=Ta_o;
for n=rocklev+2:nlev
T(:,n)=(Ta_o(:)-T(:,rocklev+1)).*zetaice(n-rocklev)+T(:,rocklev+1);
end

Tgeoth_equilib = -(Geoflux/krock)* vnodal_depths(1,1) + T(:,rocklev+1);  %calculates T at base of bedrock so that rock is initially in equilib with geoflux and initialized ice temp
for n=1:rocklev
   T(:,n)= Tgeoth_equilib(:) + (Geoflux/krock)*(vnodal_depths(1,1)-vnodal_depths(1,n));
end	

Ta=zeros(nods,1);
Tai=zeros(nods,1);
Tai=Tsurface;
%Ts=zeros(nods,1);
%Z=zeros(nods,1);
Zs=zeros(nods,1);



del18oxyicecore=zeros((fix(9997/2)),2);
del18oxydeepsea=zeros(5021,2);
load gripdatabar.mat
load specmap.mat
specmap=sealeveldata;

%clear theta

Tbar=zeros(nel,icelev);
Tpmpbar=zeros(nel,icelev);
Tpmp=zeros(nods,icelev);
THbar=zeros(nel,icelev);
polythermal=zeros(icelev,nods);
T_tentative=zeros(nlev,1);

fluxbc=zeros(nods,1);
snow=zeros(nods*2,1);
soup=zeros(nods*2,1);
water=zeros(nods*2,1);
till=zeros(nods*2,1);
Sdot=zeros(nods*2,1);
soupdot=zeros(nods*2,1);
sxbar=zeros(nel,1);
PDDbar=zeros(nel,1);


w=zeros(nods,icelev);




history=zeros(nsteps,8);
nodehistory=zeros(nsteps,8);
CO2warming=0;

snapcount=0;
sstime=0;
firstsstime=1;  %logic to catch first time vol steadystate is reached.
simulatedtime=0;

